/*
* Copyright (c) 2005-08, Craig Bishop and Colin Graeme Johnson
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the names of Craig Bishop and/or Colin Graeme Johnson, 
*       nor the name of the University of Kent may be used to endorse 
*       or promote products derived from this software without
*       specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY COLIN GRAEME JOHNSON ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL COLIN GRAEME JOHNSON BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * Created on 12-Jul-2005
 */
package main.rules;

/**
 * @author cbishop
 */
public class ResultStringer {

	private String variable;

	/**
	 * Constructor for ResultStringer
	 * 
	 * @param var
	 *            Variable name
	 */
	public ResultStringer(String var) {
		variable = var;
	}

	/**
	 * Return statement to highlight where no offending statement has been
	 * explicitly detected
	 * 
	 * @param role
	 *            String being role declared for variable in source code
	 * @return String
	 */
	public String offendingStatement(String role) {
		return variable + "%%" + role;
	}

	/**
	 * Return string stating that variable appears to be fixed value
	 * 
	 * @return String
	 */
	public String fixedValue() {
		return "variable " + variable + " appears to be 'fixed value'";
	}

	/**
	 * Return string stating that variable appears to be organizer
	 * 
	 * @return String
	 */
	public String organizer() {
		return "variable " + variable + " appears to be 'organizer'";
	}

	/**
	 * Return string stating that variable appears to be stepper
	 * 
	 * @return String
	 */
	public String stepper() {
		return "variable " + variable + " appears to be 'stepper'";
	}

	/**
	 * Return string stating that variable appears to be most recent holder
	 * 
	 * @return String
	 */
	public String mostRecentHolder() {
		return "variable " + variable + " appears to be 'most recent holder'";
	}

	/**
	 * Return string stating that variable appears to be gatherer
	 * 
	 * @return String
	 */
	public String gatherer() {
		return "variable " + variable + " appears to be 'gatherer'";
	}

	/**
	 * Return string stating that variable appears to be most wanted holder
	 * 
	 * @return String
	 */
	public String mostWantedHolder() {
		return "variable " + variable + " appears to be 'most wanted holder'";
	}

	/**
	 * Return string stating that variable appears to be one way flag
	 * 
	 * @return String
	 */
	public String oneWayFlag() {
		return "variable " + variable + " appears to be 'one way flag'";
	}

	/**
	 * Return string stating that variable appears to be transformation
	 * 
	 * @return String
	 */
	public String transformation() {
		return "variable " + variable + " appears to be 'transformation'";
	}

	/**
	 * Return string stating that variable appears to be follower
	 * 
	 * @return String
	 */
	public String follower() {
		return "variable " + variable + " appears to be 'follower'";
	}

	/**
	 * Return string stating that variable appears to be temporary
	 * 
	 * @return String
	 */
	public String temporary() {
		return "variable " + variable + " appears to be 'temporary'";
	}

	/**
	 * Return string stating that variable appears to have other role
	 * 
	 * @return String
	 */
	public String other() {
		return "not clear which role variable " + variable + " is playing";
	}
}