/*
* Copyright (c) 2005-08, Craig Bishop and Colin Graeme Johnson
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the names of Craig Bishop and/or Colin Graeme Johnson, 
*       nor the name of the University of Kent may be used to endorse 
*       or promote products derived from this software without
*       specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY COLIN GRAEME JOHNSON ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL COLIN GRAEME JOHNSON BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * Created on 20-Jun-2005
 */
package main.rules;

import java.util.ArrayList;
import java.util.LinkedHashMap;

/**
 * @author cbishop
 */
public class Stepper extends RoleChecker {
	/**
	 * Constructor for Stepper
	 * 
	 * @param analysedMap
	 *            LinkedHashMap of analysed statements
	 * @param variable
	 *            String being variable name
	 * @param methods
	 *            ArrayList of method names
	 */
	public Stepper(LinkedHashMap analysedMap, String variable, ArrayList methods) {
		super(analysedMap, variable, methods);
	}

	/**
	 * Return list of result from role check
	 * 
	 * @return ArrayList
	 */
	public ArrayList checkRole() {
		ArrayList results = stepper();
		if (!isStepper()) {
			if (isFixedValue()) {
				results.add(resultStringer.fixedValue());
			} else if (isOrganizer()) {
				results.add(resultStringer.organizer());
			} else if (isMostRecentHolder()) {
				results.add(resultStringer.mostRecentHolder());
			} else if (isGatherer()) {
				results.add(resultStringer.gatherer());
			} else if (isMostWantedHolder()) {
				results.add(resultStringer.mostWantedHolder());
			} else if (isOneWayFlag()) {
				results.add(resultStringer.oneWayFlag());
			} else if (isTransformation()) {
				results.add(resultStringer.transformation());
			} else if (isFollower()) {
				results.add(resultStringer.follower());
			} else if (isTemporary()) {
				results.add(resultStringer.temporary());
			} else {
				results.add(resultStringer.other());
			}
		}
		return results;
	}
}

/*
 * debug rules for stepper
 * 
 * if (!incDecStatement) { results.add(debugStringer.incDec()+ " and "); } if
 * (!isAssignedInFor) { results.add(debugStringer.assignedInFor(false) + " and
 * "); } if (!toggleStatement) { results.add(debugStringer.toggle() + " and "); }
 * if (!nestedBooleanAssign) { results.add(debugStringer.nestedBooleanAssign()); }
 * if (!incDecStatement && !isAssignedInFor && !toggleStatement &&
 * !nestedBooleanAssign) { result = false; } results.add(0, new
 * Boolean(result)); return results;
 */
